# coding: utf-8
# author: soonyenju@outlook.com
# date:11/28/2019

from osgeo import gdal, osr, ogr
import os
import numpy as np
try:
	from pyproj import Proj, Geod, transform
except ImportError:
	Proj, Geod, transform = None, None, None
	print("No module named pyproj")

class Raster(object):
    """docstring for Raster"""
    def __init__(self, raster_path):
        super(Raster, self).__init__()
        gdal.SetConfigOption('GDAL_FILENAME_IS_UTF8', 'NO')
        gdal.SetConfigOption('SHAPE_ENCODING', 'gb2312')
        try:
            self.path = raster_path.as_posix()
        except Exception as e:
            print(e)
            self.path = raster_path
        
    def __del__(self):
        pass

    def read(self):
        src = gdal.Open(self.path)
        cols = src.RasterXSize
        rows = src.RasterYSize
        bands = src.RasterCount
        proj = src.GetProjection()
        gt = src.GetGeoTransform()
        if bands == 1:
            array = src.ReadAsArray(0, 0, cols, rows)
        else:
            try:
                array = src.GetVirtualMemArray()
            except Exception as e:
                print(e)
                array = []
                for band in np.arange(bands) + 1:
                    band_raster = src.GetRasterBand(band)
                    data = band_raster.ReadAsArray(0, 0, cols, rows)
                    array.append(data)
                array = np.array(array)
        return {"data": array, "info": {"gt": gt, "proj": proj, "count": bands}}

    def read_info(self):
        src = gdal.Open(self.path)
        files = src.GetFileList()
        meta = src.GetMetadata_List()
        sds = src.GetSubDatasets()
        cols = src.RasterXSize
        rows = src.RasterYSize
        bands = src.RasterCount
        gt = src.GetGeoTransform()
        proj = src.GetProjection()
        return {"FileLists": files, "metadata": meta, "subdatasets": sds,
                "shape": [rows, cols, bands], "gt": gt, "proj": [proj]}

    def read_all(self):
        data = self.read()["data"]
        info = self.read_info()
        return {"data": data, "info": info}

    def write(self, array, fulloutpath, gt = "OMIGLOBAL025", EPSGPROJCODE = 4326, datatype=gdal.GDT_Float32):
        """
        parameter: array, fulloutpath, gt = "OMIGLOBAL025", datatype = gdal.GDT_Float32
        return: NONE
        datatype: gdal.Byte, gdal.GDT_UInt16, gdal.GDT_Float32
        """
        if gt == "OMIGLOBAL025":
            cols = 1440
            rows = 720
            originX = -179.88
            originY = -89.88
            pixelWidth = 0.25
            pixelHeight = 0.25
        else:
            cols = array.shape[1]
            rows = array.shape[0]
            originX = gt[0]
            originY = gt[3]
            pixelWidth = gt[1]
            pixelHeight = gt[5]

        if len(array.shape) == 2:
            array = array[np.newaxis, :, :]
        bands = array.shape[0]

        # start writing from here.
        dr = gdal.GetDriverByName('GTiff') # register driver
        ds = dr.Create(fulloutpath, cols, rows, 1, datatype)
        ds.SetGeoTransform((originX, pixelWidth, 0, originY, 0, pixelHeight))
        # or you can do: ds.SetGeoTransform(gt)
        # setting the projection
        srs = osr.SpatialReference()
        try:
            srs.ImportFromEPSG(int(EPSGPROJCODE))
        except Exception as e:
            print(Exception, ":", e, " Can not found gcs.csv")
            p = Proj(init="epsg:" + str(EPSGPROJCODE))
            srs.ImportFromProj4(p.srs)
        ds.SetProjection(srs.ExportToWkt())

        for band in range(bands):
            outband = ds.GetRasterBand(band + 1)
            outband.WriteArray(array[band, :, :])
            outband.SetNoDataValue(-9999) # NOTICE: maybe not work
            outband.FlushCache()
        del(ds)


class Vector(object):
    """docstring for Vector"""

    def __init__(self, vector_path):
        super(Vector, self).__init__()
        gdal.SetConfigOption('GDAL_FILENAME_IS_UTF8', 'NO')
        gdal.SetConfigOption('SHAPE_ENCODING', 'gb2312')
        try:
            self.path = vector_path.as_posix()
        except Exception as e:
            print(e)
            self.path = vector_path

    def __del__(self):
        pass

    def read(self):
        ds = ogr.Open(self.path)
        count = ds.GetLayerCount()
        # print(dir(ds))
        # print(count)
        dataset = {}
        for lyrNum in range(count):
            layer = ds.GetLayer(lyrNum)
            layerDefn = layer.GetLayerDefn()
            spatialRef = layer.GetSpatialRef()  # spatial reference
            geomType = layerDefn.GetGeomType()  # geometry object type wkbPoint, wkbLineString, wkbPolygon
            # print(layerDefn, layerDefn.GetFieldCount())
            field_attrs = []
            field_names = []
            for fldNum in range(layerDefn.GetFieldCount()):
                field = layerDefn.GetFieldDefn(fldNum)
                # print(field.GetName())
                # print(dir(field))
                fieldName = field.GetName()
                fieldTypeCode = field.GetType()
                fieldType = field.GetFieldTypeName(fieldTypeCode)
                fieldWidth = field.GetWidth()
                GetPrecision = field.GetPrecision()
                field_names.append(fieldName)
                field_attrs.append({"fieldTypeCode": fieldTypeCode,
                                    "fieldType": fieldType,
                                    "fieldWidth": fieldWidth,
                                    "GetPrecision": GetPrecision})
            geometries = []
            for feature in layer:
                geometry = feature.GetGeometryRef().ExportToWkt()
                # print(geometry)
                # geometries.append(geometry)
                record = {}
                for fldNum in range(layerDefn.GetFieldCount()):
                    field_name = field_names[fldNum]
                    field_attr = field_attrs[fldNum]
                    fieldval = feature.GetField(field_name)
                    record[field_name] = {"val": fieldval, "attr": field_attr}
                geometries.append({"geometry": geometry, "attrlist": record})
                # print(dir(feature))
                # print(records)
                # # record = [feature.GetField(fldMeta["fieldName"]) for fldMeta in field_metas]
                # # print(record)
                # exit(0)
            dataset[layer.GetName()] = geometries
            dataset["spatialReference"] = spatialRef
            dataset["geometryType"] = geomType
        ds.Destroy()
        return dataset

    def write(self, dataset, fulloutpath, SPATIALREF = None, EPSGCODE = 4326, GEOMTYPE = None):
        # 有Bug,写出的shp在ArcGIS中不识别
        gdal.SetConfigOption('GDAL_FILENAME_IS_UTF8', 'NO')  # 解决中文路径
        gdal.SetConfigOption('SHAPE_ENCODING', 'gb2312')  # 解决 SHAPE 文件的属性值
        driver = ogr.GetDriverByName("ESRI Shapefile")
        if os.access(fulloutpath, os.F_OK):  # 如文件已存在，则删除
            driver.DeleteDataSource(fulloutpath)
        ds = driver.CreateDataSource(fulloutpath)  # 创建 Shape文件

        try:
            spatialRef = dataset["spatialReference"]
            dataset.pop("spatialReference")
            geomtype = dataset["geometryType"]
            dataset.pop("geometryType")
        except Exception as e:
            print(e)
            if SPATIALREF:
                spatialRef = SPATIALREF
            else:
                spatialRef = osr.SpatialReference()
                spatialRef.ImportFromEPSG(int(EPSGCODE))
            if GEOMTYPE:
                geomtype = GEOMTYPE
            else:
                geomtype = ogr.wkbPolygon
        for layerName, geometries in dataset.items():
            # print(layerName)
            layer = ds.CreateLayer(layerName, srs = spatialRef, geom_type = geomtype)  # 创建图层
            for geometryDict in geometries:
                # print(geometryDict)
                attrlist = geometryDict["attrlist"]
                for fieldName, meta in attrlist.items():
                    try:
                        field = ogr.FieldDefn(
                            fieldName, self.ogr_type_code_mapping(meta["attr"]["fieldTypeCode"]))
                    except Exception as e:
                        try:
                            field = ogr.FieldDefn(
                                fieldName, ogr.OFTReal)
                        except Exception as e:
                            field = ogr.FieldDefn(
                                fieldName, ogr.OFTString)

                    if "fieldWidth" in meta["attr"].keys():
                        field.SetWidth(meta["attr"]["fieldWidth"])
                    if "GetPrecision" in meta["attr"].keys():
                        field.SetPrecision(meta["attr"]["GetPrecision"])
                    # if "fieldType" in meta["attr"].keys():
                    # 	field.SetType(meta["attr"]["fieldTypeCode"])
                    field.SetName(fieldName)
                    layer.CreateField(field)

                geomtry = ogr.CreateGeometryFromWkt(geometryDict["geometry"])
                feature = ogr.Feature(layer.GetLayerDefn())  # 创建 SF
                feature.SetGeometry(geomtry)
                for fieldName, meta in attrlist.items():
                    feature.SetField(fieldName, meta["val"])
                layer.CreateFeature(feature)  # 将 SF 写入图层


        ds.Destroy()  # 关闭文件



        # for fd in fieldlist:  # 将字段列表写入图层
        # field = ogr.FieldDefn(fd['name'], fd['type'])
        # if fd.has_key('width'):
        # field.SetWidth(fd['width'])
        # if fd.has_key('decimal'):
        # field.SetPrecision(fd['decimal'])
        # layer.CreateField(field)
        # for i in range(len(reclist)):  # 将 SF 数据记录（几何对象及其属性写入图层）
        # geom = ogr.CreateGeometryFromWkt(geomlist[i])
        # feat = ogr.Feature(layer.GetLayerDefn())  # 创建 SF
        # feat.SetGeometry(geom)
        # for fd in fieldlist:
        # feat.SetField(fd['name'], reclist[i][fd['name']])
        # layer.CreateFeature(feat)  # 将 SF 写入图层
        # ds.Destroy()  # 关闭文件

    def ogr_type_code_mapping(self, type_code):
        # OFTInteger = 0, OFTIntegerList = 1, OFTReal = 2, OFTRealList = 3,
        # OFTString = 4, OFTStringList = 5, OFTWideString = 6, OFTWideStringList = 7,
        # OFTBinary = 8, OFTDate = 9, OFTTime = 10, OFTDateTime = 11
        ogr_type_dict = {
            "0": ogr.OFTInteger,
            "1": ogr.OFTIntegerList,
            "2": ogr.OFTReal,
            "3": ogr.OFTRealList,
            "4": ogr.OFTString,
            "5": ogr.OFTStringList,
            "6": ogr.OFTWideString,
            "7": ogr.OFTWideStringList,
            "8": ogr.OFTBinary,
            "9": ogr.OFTDate,
            "10": ogr.OFTTime,
            "11": ogr.OFTDateTime
        }

        return ogr_type_dict[str(type_code)]



import math
import datetime

class Sunriseset:
    def __init__(self, timestamp = None, format = r"%Y-%m-%d"):
        if isinstance(timestamp, str):
            timestamp = datetime.datetime.strptime(timestamp, format)
        self.timestamp = timestamp

    def __call__(self, lon, lat):
        coords = {'longitude' : lon, 'latitude' : lat}

        # Sunrise time UTC (decimal, 24 hour format)
        sunrise = self.getSunriseTime(coords)['decimal']

        # Sunset time UTC (decimal, 24 hour format)
        sunset = self.getSunsetTime(coords)['decimal']
        return {
            "sunrise": sunrise,
            "sunset": sunset
        }

    def getSunriseTime(self, coords):
        return self.calcSunTime(coords, True)

    def getSunsetTime(self, coords):
        return self.calcSunTime(coords, False)

    def getCurrentUTC(self):
        now = datetime.datetime.now()
        return [now.day, now.month, now.year]

    def calcSunTime(self, coords, isRiseTime, zenith = 90.8):
        # isRiseTime == False, returns sunsetTime
        if self.timestamp:
            timestamp = self.timestamp
            try:
                day, month, year = [timestamp.day, timestamp.month, timestamp.year]
            except:
                day, month, year = timestamp
#             else:
#                 raise Exception("Wrong input time format...")
        else:
            print("Use current time...")
            day, month, year = self.getCurrentUTC()

        longitude = coords['longitude']
        latitude = coords['latitude']

        TO_RAD = math.pi/180

        #1. first calculate the day of the year
        N1 = math.floor(275 * month / 9)
        N2 = math.floor((month + 9) / 12)
        N3 = (1 + math.floor((year - 4 * math.floor(year / 4) + 2) / 3))
        N = N1 - (N2 * N3) + day - 30

        #2. convert the longitude to hour value and calculate an approximate time
        lngHour = longitude / 15

        if isRiseTime:
            t = N + ((6 - lngHour) / 24)
        else: #sunset
            t = N + ((18 - lngHour) / 24)

        #3. calculate the Sun's mean anomaly
        M = (0.9856 * t) - 3.289

        #4. calculate the Sun's true longitude
        L = M + (1.916 * math.sin(TO_RAD*M)) + (0.020 * math.sin(TO_RAD * 2 * M)) + 282.634
        L = self.forceRange( L, 360 ) #NOTE: L adjusted into the range [0,360)

        #5a. calculate the Sun's right ascension

        RA = (1/TO_RAD) * math.atan(0.91764 * math.tan(TO_RAD*L))
        RA = self.forceRange( RA, 360 ) #NOTE: RA adjusted into the range [0,360)

        #5b. right ascension value needs to be in the same quadrant as L
        Lquadrant  = (math.floor( L/90)) * 90
        RAquadrant = (math.floor(RA/90)) * 90
        RA = RA + (Lquadrant - RAquadrant)

        #5c. right ascension value needs to be converted into hours
        RA = RA / 15

        #6. calculate the Sun's declination
        sinDec = 0.39782 * math.sin(TO_RAD*L)
        cosDec = math.cos(math.asin(sinDec))

        #7a. calculate the Sun's local hour angle
        cosH = (math.cos(TO_RAD*zenith) - (sinDec * math.sin(TO_RAD*latitude))) / (cosDec * math.cos(TO_RAD*latitude))

        if cosH > 1:
            return {'status': False, 'msg': 'the sun never rises on this location (on the specified date)'}

        if cosH < -1:
            return {'status': False, 'msg': 'the sun never sets on this location (on the specified date)'}

        #7b. finish calculating H and convert into hours

        if isRiseTime:
            H = 360 - (1/TO_RAD) * math.acos(cosH)
        else: #setting
            H = (1/TO_RAD) * math.acos(cosH)

        H = H / 15

        #8. calculate local mean time of rising/setting
        T = H + RA - (0.06571 * t) - 6.622

        #9. adjust back to UTC
        UT = T - lngHour
        UT = self.forceRange( UT, 24) # UTC time in decimal format (e.g. 23.23)

        #10. Return
        hr = self.forceRange(int(UT), 24)
        min = round((UT - int(UT))*60,0)

        return {
            'status': True,
            'decimal': UT,
            'hr': hr,
            'min': min 
        }

    def forceRange(self, v, max):
        # force v to be >= 0 and < max
        if v < 0:
            return v + max
        elif v >= max:
            return v - max

        return v

'''
# example:
if __name__ == "__main__":
    srs = Sunriseset("2018-01-01")
    print(srs(0, 50))
'''


import numpy as np
import matplotlib.pyplot as plt
from scipy import ndimage
from scipy.interpolate import griddata
from scipy.interpolate import Rbf, LinearNDInterpolator, interp2d
from scipy.spatial import cKDTree as KDTree
from mpl_toolkits.axes_grid1 import make_axes_locatable

class IDW(object):
    """ 
    # https://mail.python.org/pipermail/scipy-user/2010-June/025920.html
    # https://github.com/soonyenju/pysy/blob/master/pysy/scigeo.py
    inverse-distance-weighted interpolation using KDTree:
    invdisttree = Invdisttree(X, z)  
    -- points, values
    interpol = invdisttree(q, k=6, eps=0)
    -- interpolate z from the 6 points nearest each q;
        q may be one point, or a batch of points

    """
    def __init__(self, X, z, leafsize = 10):
        super()
        self.tree = KDTree(X, leafsize=leafsize)  # build the tree
        self.z = z

    def __call__(self, q, k = 8, eps = 0):
        # q is coor pairs like [[lon1, lat1], [lon2, lat2], [lon3, lat3]]
        # k nearest neighbours of each query point --
        # format q if only 1d coor pair passed like [lon1, lat1]
        if not isinstance(q, np.ndarray):
            q = np.array(q)
        if q.ndim == 1:
            q = q[np.newaxis, :]

        self.distances, self.ix = self.tree.query(q, k = k,eps = eps)
        interpol = []  # np.zeros((len(self.distances),) +np.shape(z[0]))
        for dist, ix in zip(self.distances, self.ix):
            if dist[0] > 1e-10:
                w = 1 / dist
                wz = np.dot(w, self.z[ix]) / np.sum(w)  # weightz s by 1/dist
            else:
                wz = self.z[ix[0]]
            interpol.append(wz)
        return interpol

def gen_buffer(lon, lat, step, shape = "rectangle"):
    if shape == "rectangle":
        # clockwise
        coors = [
                 [lon - step, lat + step], # upper left
                 [lon + step, lat + step], # upper right
                 [lon + step, lat - step], # lower right
                 [lon - step, lat - step], # lower left
        ]

    return coors

def dms2ddm(deg, min_, sec):
    # covert Degrees Minutes Seconds (DMS) to Degrees Decimal Minutes (DDM)
    min_ = min_ + sec / 60
    ddm = deg + min_ / 60
    
    return ddm

def deg2km(lat):
    # earth radius: 6371 km
    return 6371 * np.cos(lat) * 2* np.pi / 360

def grid2points(arr, lons, lats, lon_pnts, lat_pnts, order = 1):
    # Python array starts from upper left (longitude ascending but latitude descending)
    # check orders:
    if lats[-1] > lats[0]:
        lats = lats[::-1]
        arr = arr[::-1, :]
    if lons[-1] < lons[0]:
        lons = lons[::-1]
        arr = arr[:, ::-1]
    # print(lats)
    # print(lons)
    # print(arr)
    def lonlat2xy(lonlats, lonlat):
        xy = (lonlat - lonlats[0]) / (lonlats[-1] - lonlats[0]) * (len(lonlats) - 1)
        return xy
    
    map_lons = [lonlat2xy(lons, lon) for lon in lon_pnts]
    map_lats = [lonlat2xy(lats, lat) for lat in lat_pnts]
    # print(map_lons)
    # print(map_lats)

    # map_lat = (lat - lats[0]) / (lats[-1] - lats[0]) * (len(lats) - 1)
    # # print(map_lat)

    # map_lon = (lon - lons[0]) / (lons[-1] - lons[0]) * (len(lons) - 1)
    # # print(map_lon)

    return ndimage.map_coordinates(arr, [map_lats, map_lons], order = order)

def grid2points2(arr, lons, lats, lon_pnts, lat_pnts, missing = -9999, method = 'linear'):
    if missing: arr[np.where(arr == missing)] = np.nan
    lonlon, latlat = np.meshgrid(lons, lats)
    data = griddata((latlat.ravel(), lonlon.ravel()), arr.ravel(), (lat_pnts, lon_pnts), method = method)
    return data

def points2grid(data, lons, lats, missing = -9999, method = 'nearest'):
    uniqueLats = np.unique(lats)
    uniqueLons = np.unique(lons)
    if uniqueLats[0] < uniqueLats[-1]: uniqueLats = np.flip(uniqueLats) 
    # uniqueLats = uniqueLats[::-1] # lats shoud be descending from upper to bottom

    if missing: data[np.where(data == missing)] = np.nan
    points = list(zip(lons, lats))
    grid_x, grid_y = np.meshgrid(uniqueLons, uniqueLats)
    arr = griddata(points, data, (grid_x, grid_y), method = method)

    return arr, uniqueLons, uniqueLats

def points2points(data, lons_orig, lats_orig, lons_tar, lats_tar):
    """
    data, lons_orig, lats_orig, lons_tar, lats_tar are all vectors (1D array)
    """
    interp = LinearNDInterpolator(list(zip(lons_orig, lats_orig)), data)
    # interp = Rbf(lons_orig, lats_orig, arr,function='linear')
    return interp(lons_tar, lats_tar)

def latex_float(f):
    if (np.abs(f) != 0) & ((np.abs(f) < 0.01) or (np.abs(f) > 100)):
        float_str = "{0:.2e}".format(f)
    else:
        float_str = "{0:.2g}".format(f)
    if "e" in float_str:
        base, exponent = float_str.split("e")
        return r"${0} \times 10^{{{1}}}$".format(base, int(exponent))
    else:
        return float_str

def get_stats(arr, n = 5):
    min_ = np.nanmin(arr)
    max_ = np.nanmax(arr)
    interval = (max_ - min_) / (n - 1)
    ticks = [min_ + i * interval for i in range(n)]
    return ticks

def map2darr(arr, uniqueLons, uniqueLats, unit = "", cmap = "viridis", figsize = [6, 4]):
    ticklabels = [latex_float(f) for f in get_stats(arr)]

    arr = 255 * (arr - np.nanmin(arr)) / (np.nanmax(arr) - np.nanmin(arr))

    ticks = get_stats(arr)

    fig, ax = plt.subplots(figsize = figsize)
    divider = make_axes_locatable(ax)
    cax = divider.append_axes('right', size='5%', pad=0.05)
    im = ax.imshow(arr, cmap = cmap, extent = [uniqueLons.min(), uniqueLons.max(), uniqueLats.min(), uniqueLats.max()])
    # Add colorbar, make sure to specify tick locations to match desired ticklabels
    cbar = fig.colorbar(im, cax = cax, orientation = 'vertical', ticks = ticks)
    cbar.ax.set_yticklabels(ticklabels)  # vertically oriented colorbar

    cbar.set_label(unit)

    ax.set_xlabel("Longitude")
    ax.set_ylabel("Latitude")

    return fig, ax


# coding: utf-8
# author: soonyenju@outlook.com
# date: 11/28/2019

from pathlib import Path
from shapely.geometry import Polygon
import rasterio as rio
from rasterio.mask import mask
from rasterio.enums import Resampling
import geopandas as gpd
import warnings
import numpy as np

class Raster(object):
    """
    the wrapper of rasterio
    """
    def __init__(self, path):
        super() # or: super(Raster, self).__init__()
        self.path = path

    def __del__(self):
        if hasattr(self, "src"):
            self.src.closed

    def read(self):
        warnings.filterwarnings("ignore")
        with rio.open(self.path) as src:
            array = src.read()
            profile = src.profile
        return {"array": array, "meta": profile}

    def fopen(self, src_only = True):
        """
        c, a, b, f, d, e = src.transform
        gt = rasterio.transform.Affine.from_gdal(c, a, b, f, d, e)
        proj = src.crs
        count = src.count
        name = src.name
        mode = src.mode
        closed = src.closed
        width = src.width
        height = src.height
        bounds = src.bounds
        idtypes = {i: dtype for i, dtype in zip(
            src.indexes, src.dtypes)}
        meta = src.meta
        src = src.affine
        """
        self.src = rio.open(self.path)
        if not src_only:
            self.data = self.src.read()
            self.profile = self.src.profile

    def write(self, array, fulloutpath, profile, dtype = rio.float64):
        count=profile["count"]
        # bug fix, can't write a 3D array having a shape like (1, *, *)
        if array.ndim == 3 and array.shape[0] == 1:
            array = array[0, :, :]
        profile.update(dtype = dtype, count = count, compress='lzw')
        with rio.open(fulloutpath, 'w', **profile) as dst:
            dst.write(array.astype(dtype), count)

    def clip(self, polygon):
        self.clip_arr, self.clip_transform = mask(self.src, polygon, crop=True)

    def resampling(self, new_shape):
        """
        new_shape format: height, width, count in order
        Resample: default Resampling.bilinear, other choice includes Resampling.average
        """
        height, width, count = new_shape
        resampled_data = self.src.read(
            out_shape = (height, width, count),
            resampling = Resampling.bilinear
        )
        return resampled_data

    def get_pntpairs(self, **kwargs):
        # compatible with affine format, rather than geotransform
        if not kwargs:
            # print("Inside data")
            affine = self.profile["transform"]
            cols = self.profile["width"]
            rows = self.profile["height"]
            data = self.data.ravel()
        else:
            # print("Outside data")
            # NOTICE: try to transform the geotransform to affine.
            affine = kwargs["affine"]
            # NOTICE: the first dimension of rasterio array is band.
            cols = kwargs["data"].shape[2]
            rows = kwargs["data"].shape[1]
            data = kwargs["data"].ravel()
        # print(affine)
        # print(profile)
        lats = [idx * affine[4] + affine[5] for idx in range(rows)]
        lons = [idx * affine[0] + affine[2] for idx in range(cols)]

        lons, lats = np.meshgrid(lons, lats)

        pntpairs = np.vstack([lons.ravel(), lats.ravel()]).T
        return pntpairs, data

class Vector(object):
    """docstring for Vector"""

    def __init__(self, **kwargs):
        super(Vector, self).__init__()
        if "path" in kwargs.keys():
            vector_path = kwargs["path"]
            try:
                self.path = vector_path.as_posix()
            except Exception as e:
                print(e)
                self.path = vector_path

    def __del__(self):
        pass

    def read(self):
        gdf = gpd.read_file(self.path)
        return gdf

    def write(self, gpd, fulloutpath):
        # filetype = fulloutpath.split('.')[-1]
        gpd.to_file(fulloutpath)

    def create_polygon(self, coors, epsg_code = "4326"):
        polygon_geom = Polygon(coors)
        crs = {"init": "epsg:" + epsg_code}
        poly = gpd.GeoDataFrame(index=[0], crs=crs, geometry=[polygon_geom])  # gdf   
        # gjs = poly.to_json()
        return poly


#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: Songyan Zhu
# Mail: zhusy93@gmail.com
# Created Time:  2018-10-23 13:28:34
#############################################


from setuptools import setup, find_packages

setup(
	name = "scigeo",
	version = "0.0.7",
	keywords = ("easy geo warpper"),
	description = "read/dump/process geo data",
	long_description = "coming soon",
	license = "MIT Licence",

	url="https://github.com/soonyenju/scigeo",
	author = "Songyan Zhu",
	author_email = "zhusy93@gmail.com",

	packages = find_packages(),
	include_package_data = True,
	platforms = "any",
	install_requires=[
            # "geopandas==0.4.1",
            # "scipy==1.2.1",
            # "GDAL==2.3.3",
            # "pyproj==1.9.6",
            # "Shapely==1.6.4.post1",
            # "rasterio==1.0.21",
            # "numpy==1.16.0",
			"geopandas",
			"rasterio"

	]
)